import json
import logging
import requests
import traceback
from .az import AzureExtras
from time import time, sleep


class StreamAnalyticsJobs(AzureExtras):
    def __init__(self, path, rg):
        super().__init__(path, rg)
        self.url = f"{self.url}/providers/Microsoft.StreamAnalytics/streamingjobs"

    def get_stream_analytics_job(self, job):
        url = f"{self.url}/{job}"
        params = {
            "api-version": "2015-10-01",
            "$expand": "inputs,transformation,outputs,functions",
        }
        try:
            response = requests.get(url, headers=self.headers, params=params)
            body = json.loads(response.content if response.content else "{}")
            logging.debug(json.dumps(body))
            if response.ok is False:
                raise AssertionError(
                    f"Failed to get {job} status from {url}\n"
                    + f"Response Code: {response.status_code}\n"
                    + f"Response Reason: {response.reason}"
                )
        except Exception as error:
            logging.debug(traceback.format_exc())
            raise error

        return body

    def toggle_stream_analytics_job(self, job, action):
        url = f"{self.url}/{job}/{action}"
        params = {"api-version": "2015-10-01"}

        try:
            response = requests.post(url, headers=self.headers, params=params)
            if response.ok is False:
                raise AssertionError(
                    f"Failed to {action} {job} using {url}\n"
                    + f"Response Code: {response.status_code}\n"
                    + f"Response Reason: {response.reason}"
                )

            logging.info(f"Sent {action} to {job}")
            timeout = time() + 120
            while time() < timeout:
                logging.info(f"Checking status of {action} sent to {job}..")
                results = self.get_stream_analytics_job(job)
                status = results["properties"]["jobState"]
                logging.debug(f"RESULTS: {status}")
                started = action == "start" and status == "Running"
                stopped = action == "stop" and status == "Stopped"
                if started or stopped:
                    logging.info(f"Successfully sent {action} to {job}.")
                    return results
                logging.info(f"Waiting 10 seconds before checking again...")
                sleep(10)

            raise AssertionError(
                f"Failed to {action} {job}. Timed out. Status: {status}"
            )
        except Exception as error:
            logging.debug(traceback.format_exc())
            raise error



import json
import logging
import requests
import traceback
from .app import AppService
from time import time


class KuduClient(AppService):
    """
    https://github.com/projectkudu/kudu/wiki/REST-API
    """

    def __init__(self, path, rg, app):
        super().__init__(path, rg)
        self.app = app
        self.url = self.get_publishing_credentials(app)["scmUri"] + "/api/"
        logging.debug(self.url)

    def get_logs(self, line_count):
        # response = requests.get(self.url + "logs/recent")
        response = requests.get(f"{self.url}vfs/LogFiles/application")
        logfiles = response.json()
        sorted_logfiles = sorted(logfiles, key=lambda d: d["mtime"])
        if len(sorted_logfiles) < 1:
            print(f"No logfiles found on {self.app} :-(")
            return
        latest_logfile = sorted_logfiles[len(sorted_logfiles) - 1]["name"]
        response = requests.get(f"{self.url}vfs/LogFiles/application/{latest_logfile}")
        response_lines = response.text.split("\r\n")
        print("\n".join(response_lines[-line_count:]))

    def deploy_zip(self, path):
        """
        https://github.com/projectkudu/kudu/wiki/REST-API#zip-deployment

        Deploy from zip asynchronously. The Location header of the response
        will contain a link to a pollable deployment status.
        """
        try:
            with open(path, "rb") as f:
                zipfile = f.read()

            response = requests.put(self.url + "zipdeploy?isAsync=true", data=zipfile)

            if response.ok:
                timeout = time() + 60
                while time() < timeout:
                    logging.debug(f"Checking deployment {response.headers['Location']}")
                    status = requests.get(
                        response.headers["Location"], headers=self.headers
                    )
                    logging.debug(f"Deployment Complete: {status.json()['complete']}")
                    if status.json()["complete"]:
                        return status.json()

            raise AssertionError(
                f"Deploying {path} on {self.url}zipdeploy\n"
                + f"Code:{response.status_code}, Message: {response.text}"
            )
        except Exception as error:
            logging.debug(f"Failed to deploy {path} from {self.url}zipdeploy: {error}")
            logging.debug(traceback.format_exc())
            raise error

    def download_zip(self, source, destination):
        """
        https://github.com/projectkudu/kudu/wiki/REST-API#zip

        Zip up and download the specified folder. The zip doesn't include the
        top folder itself. Make sure you include the trailing slash!
        """
        try:
            response = requests.get(f"{self.url}zip/{source}")
            logging.debug(response.headers)
            zipbytes = response.content
            zipfile = open(destination, "wb")
            zipfile.write(zipbytes)
            zipfile.close()
        except Exception as error:
            logging.debug(
                f"Failed to download zip from {self.url}zip/{source}: {error}"
            )
            logging.debug(traceback.format_exc())
            raise error

        return response

    def get_endpoint(self, endpoint):
        """
        https://github.com/projectkudu/kudu/wiki/REST-API
        """
        try:
            response = requests.get(self.url + endpoint)
            if response.ok is False:
                raise AssertionError(
                    f"Failed to get {endpoint} on {self.url}\n"
                    + f"Code:{response.status_code}\n"
                    + f"Response: {response.text}"
                )
        except Exception as error:
            logging.debug(f"Failed to get {endpoint} from {self.url}: {error}")
            logging.debug(traceback.format_exc())
            raise error

        logging.debug(
            "Output:\n" + json.dumps(response.json(), indent=2, sort_keys=True)
        )
        return response.json()

    def run_cmd(self, cmd, cwd):
        """
        https://github.com/projectkudu/kudu/wiki/REST-API#command
        """
        logging.debug(f"Running {cmd} on {self.app}/{cwd}...")
        payload = {"command": cmd, "dir": cwd}

        try:
            response = requests.post(self.url + "command", json=payload)
            if response.ok is False:
                raise AssertionError(
                    f"Failed to run {cmd} on {self.url}/{cwd}\n"
                    + f"Code:{response.status_code}\n"
                    + f"Response: {response.text}"
                )
        except Exception as error:
            logging.debug(f"Failed to run {cmd} on {self.app}/{cwd}: {error}")
            logging.debug(traceback.format_exc())
            raise error

        logging.debug(
            "Output:\n" + json.dumps(response.json(), indent=2, sort_keys=True)
        )
        return response.json()



import logging
import os
import sys
from argparse import ArgumentTypeError


def chkpath(path):
    if os.path.exists(path):
        return os.path.abspath(path)

    raise ArgumentTypeError(f"{path} does not exist.")


def mklog(verbosity):
    if verbosity > 1:
        loglevel = logging.DEBUG
        logformat = "%(asctime)s %(threadName)s %(levelname)s %(message)s"
    elif verbosity == 1:
        loglevel = logging.INFO
        logformat = "%(asctime)s %(levelname)s %(message)s"
    else:
        loglevel = logging.WARNING
        logformat = "%(levelname)s %(message)s"

    logging.basicConfig(
        # filename='',
        format=logformat,
        datefmt="%Y-%m-%d %H:%M:%S",
        level=loglevel,
        stream=sys.stdout,
    )



import json
import logging
import requests
import traceback
from time import time
from .az import AzureExtras


class AppService(AzureExtras):
    def __init__(self, path, rg):
        super().__init__(path, rg)
        self.url = f"{self.url}/providers/Microsoft.Web/sites"

    def get_publishing_credentials(self, app):
        """
        https://docs.microsoft.com/en-us/rest/api/appservice/webapps/listpublishingcredentials
        """
        url = f"{self.url}/{app}/config/publishingcredentials/list"
        params = {"api-version": "2019-08-01"}

        try:
            response = requests.post(url, headers=self.headers, params=params)
            if response.ok:
                logging.debug(json.dumps(response.json(), indent=2))
                return response.json()["properties"]
            raise AssertionError(
                f"Failed to get publishing credentials for {app}: {response.status_code}"
            )
        except Exception as error:
            logging.debug(traceback.format_exc())
            raise error

    def toggle_health_check(self, app, action):
        url = f"{self.url}/{app}/config/web"
        params = {"api-version": "2018-02-01"}

        if action == "enable":
            patch = {"properties": {"healthCheckPath": "/status/status.cshtml"}}
        elif action == "disable":
            patch = {"properties": {"healthCheckPath": "null"}}
        else:
            raise ValueError(f"{action} is invalid!")

        try:
            response = requests.patch(
                url, headers=self.headers, params=params, data=json.dumps(patch)
            )
            if response.ok is False:
                raise AssertionError(
                    f"Failed to patch {url} with {patch}, Code:{response.status_code}"
                )
            content = response.json()
            success = (
                content["properties"]["healthCheckPath"]
                == patch["properties"]["healthCheckPath"]
            )
            logging.debug(json.dumps(content, indent=2, sort_keys=True))
            if success is False:
                raise AssertionError(f"Failed to patch {url} with {patch}.")
        except requests.exceptions.ConnectionError as error:
            logging.warning("Transient connection issue. Trying again...")
            logging.debug(error)
            timeout = time() + 10
            while time() < timeout:
                return self.toggle_health_check(app, action)
        except Exception as error:
            logging.debug(traceback.format_exc())
            raise error

    def get_app_service(self, app):
        """
        Get App Service
        """
        url = f"{self.url}/{app}"
        params = {"api-version": "2019-08-01"}

        try:
            response = requests.get(url, headers=self.headers, params=params)
            if response.ok:
                return response.json()["properties"]
            raise AssertionError(f"Failed to get {app}: {response.status_code}")
        except Exception as error:
            logging.error(f"Failed to get status of {app}: {error}")
            logging.debug(traceback.format_exc())

    def list_app_service_slots(self, app):
        """
        List App Service Slots
        """
        url = f"{self.url}/{app}/slots"
        params = {"api-version": "2019-08-01"}

        try:
            response = requests.get(url, headers=self.headers, params=params)
            if response.ok:
                return response.json()
            raise AssertionError(
                f"Failed to get slots for {app}: {response.status_code}"
            )
        except Exception as error:
            logging.error(f"Failed to get slots for {app}: {error}")
            logging.debug(traceback.format_exc())

    def get_app_service_slot(self, app, slot):
        """
        Get App Service Slot
        """
        url = f"{self.url}/{app}/slots/{slot}"
        params = {"api-version": "2019-08-01"}

        try:
            response = requests.get(url, headers=self.headers, params=params)
            if response.ok:
                return response.json()["properties"]
            raise AssertionError(f"Failed to get {slot}: {response.status_code}")
        except Exception as error:
            logging.error(f"Failed to get status of {slot}: {error}")
            logging.debug(traceback.format_exc())

    def toggle_app_service(self, app, action):
        """
        START/STOP an Azure App Service
        """
        logging.debug(f"{action.capitalize()} {app}")
        url = f"{self.url}/{app}/{action}"
        params = {"api-version": "2016-08-01"}

        try:
            response = requests.post(url, headers=self.headers, params=params)
            if response.ok is False:
                raise AssertionError(
                    f"Failed to {action} {app}: {response.status_code}"
                )

            logging.info(f"Sent {action.capitalize()} to {app}")

            timeout = time() + 30
            while time() < timeout:
                status = self.get_app_service(app)["state"]
                started = status == "Running" and action == "start"
                stopped = status == "Stopped" and action == "stop"
                if started or stopped:
                    return status

            raise AssertionError(f"Failed to {action} {app}: {status}")
        except requests.exceptions.ConnectionError as error:
            logging.warning("Transient connection issue. Trying again...")
            logging.debug(error)
            timeout = time() + 10
            while time() < timeout:
                return self.toggle_app_service(app, action)
        except Exception as error:
            logging.error(f"Failed to {action} {app}: {error}")
            logging.debug(traceback.format_exc())

    def toggle_app_service_slot(self, app, slot, action):
        """
        START/STOP an Azure App Service Slot
        """
        url = f"{self.url}/{app}/slots/{slot}/{action}"
        params = {"api-version": "2016-08-01"}

        try:
            response = requests.post(url, headers=self.headers, params=params)
            if response.ok is False:
                raise AssertionError(
                    f"Failed to {action} {slot}: {response.status_code}"
                )

            logging.info(f"Sent {action} to {slot}")

            timeout = time() + 30
            while time() < timeout:
                status = self.get_app_service_slot(app, slot)["state"]
                started = status == "Running" and action == "start"
                stopped = status == "Stopped" and action == "stop"
                if started or stopped:
                    return status

            raise AssertionError(f"Failed to {action} {slot}: {status}")
        except requests.exceptions.ConnectionError as error:
            logging.warning("Transient connection issue. Trying again...")
            logging.debug(error)
            timeout = time() + 10
            while time() < timeout:
                return self.toggle_app_service_slot(app, slot, action)
        except Exception as error:
            logging.error(f"Failed to {action} {slot}: {error}")
            logging.debug(traceback.format_exc())



import json
import logging
import subprocess
import traceback
from configparser import ConfigParser
from azure.common.credentials import (
    ServicePrincipalCredentials,
    get_azure_cli_credentials,
)


def get_cmd_stdout(cmd):
    process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
    process.wait()
    data, err = process.communicate()
    return data.decode("utf-8")


class AzureExtras:
    def __init__(self, path, rg):
        self.config_path = path
        sub = self.get_subscription()
        token = self.get_access_token()
        self.headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json",
        }
        self.url = (
            f"https://management.azure.com/subscriptions/{sub}/resourceGroups/{rg}"
        )
        logging.debug("HEADERS:\n" + json.dumps(self.headers))

    def get_config(self, path):
        try:
            config = ConfigParser()
            config.read(path)
            az = config["azure"]
        except Exception as error:
            logging.error(f"Failed to retrieve config from {path}. Aborting.")
            logging.error(error)

        if az:
            logging.info(f"Found Azure configuration at {path}.")
            return az["client"], az["secret"], az["tenant"], az["sub"]

        logging.error(f"Failed to retrieve Azure details from {path}. Aborting.")

    def get_subscription(self):
        try:
            return get_azure_cli_credentials()[1]
        except Exception as error:
            logging.warning("Couldn't get subscription id from Azure CLI.")

        try:
            client, secret, tenant, sub = self.get_config(self.config_path)
            return sub
        except Exception as error:
            logging.error("Failed to get a subscription id.")

    def get_access_token(self):
        try:
            access_token = json.loads(get_cmd_stdout("az account get-access-token"))[
                "accessToken"
            ]
            logging.info("Authenticated with Azure CLI credentials.")
            return access_token
        except Exception as error:
            logging.warning("Failed to authenticate using Azure CLI credentials.")

        try:
            client, secret, tenant, sub = self.get_config(self.config_path)
            credentials = ServicePrincipalCredentials(
                client_id=client,
                secret=secret,
                tenant=tenant,
            )
            access_token = credentials.token["access_token"]
            logging.info(
                f"Authenticated with Service Principal credentials from {self.config_path}"
            )
            logging.debug(f"Access Token:\n\n{access_token}\n")
            return access_token
        except Exception as error:
            logging.debug(traceback.format_exc())
            raise error



# Copyright (c) 2018, Toby Slight. All rights reserved.
# ISC License (ISCL) - see LICENSE file for details.

import setuptools
import subprocess


def get_latest_tag():
    try:
        cmd_output = subprocess.run(
            ["git", "describe", "--tags", "--abbrev=0"], stdout=subprocess.PIPE
        )
        return cmd_output.stdout.strip().decode("utf-8")
    except EnvironmentError:
        print("Couldn't run git to get a version number for setup.py")


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="azure_extras",
    version=get_latest_tag(),
    author="Toby Slight",
    author_email="tslight@pm.me",
    description="The stuff Microsoft left out..",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/tslight/azure_extras",
    install_requires=["azure-common", "msrest", "msrestazure", "requests"],
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: ISC License (ISCL)",
        "Operating System :: OS Independent",
    ),
    entry_points={
        "console_scripts": [
            "az-asctl = azure_extras.asctl:main",
            "az-healthchkctl = azure_extras.healthchkctl:main",
            "az-kuductl = azure_extras.kuductl:main",
            "az-sajctl = azure_extras.sajctl:main",
        ],
    },
)
